#import kivy
#kivy.require('1.9.1')

from kivy.app import App
from kivy.lang import Builder
from kivy.core.window import Window
from kivy.uix.floatlayout import FloatLayout
from kivy.uix.relativelayout import RelativeLayout
from kivy.uix.gridlayout import GridLayout
from kivy.clock import Clock
from kivy.uix.image import Image
from kivy.animation import Animation
from kivy.uix.button import Button
from kivy.uix.widget import Widget
from kivy.uix.label import Label
import random

Builder.load_file('pw22.kv')




ROOM_SIZE_RATIO=15
ROOMS_IN_A_ROW = 13
ROOMS_IN_A_COL = 8
BOARD_ROWS = 2*ROOMS_IN_A_COL+1
BOARD_COLS = 2*ROOMS_IN_A_ROW+1
SPACING = 0



class Board(GridLayout):
    def __init__(self, win_size, **kwargs):
        self.source='stage{}.png'.format(str(random.randrange(1,4)))
        self.win_size = win_size
        self.room_size = self.win_size[1]/ROOM_SIZE_RATIO
        self.joint_size = self.room_size/5
        self.board_size = self.room_size*ROOMS_IN_A_ROW + self.joint_size*(ROOMS_IN_A_ROW+1) + SPACING*(BOARD_COLS-1), self.room_size*ROOMS_IN_A_COL + self.joint_size*(ROOMS_IN_A_COL+1) + SPACING*(BOARD_ROWS-1) 
        #print self.win_size,self.room_size,self.joint_size,self.board_size, float(self.board_size[0])/self.win_size[0], float(self.board_size[1])/self.win_size[1]
        
        super(Board, self).__init__(
            size_hint=(float(self.board_size[0])/self.win_size[0], float(self.board_size[1])/self.win_size[1]),
            pos_hint={'center_x': 0.5, 'center_y': 0.5},
            rows=BOARD_ROWS,
            cols=BOARD_COLS, 
            cols_minimum={r:self.joint_size+(self.room_size-self.joint_size)*(r%2) for r in range(BOARD_COLS)},
            rows_minimum={r:self.joint_size+(self.room_size-self.joint_size)*(r%2) for r in range(BOARD_ROWS)},
            spacing=SPACING,
            **kwargs)   
        
        self.doors=[]
        self.matrix=[]

        for row in range(BOARD_ROWS):
            matrix_row=[]
            for col in range(BOARD_COLS):
                if row==0 or row==BOARD_ROWS-1 or col==0 or col==BOARD_COLS-1:
                    t = Wall(row, col)
                elif row%2==0 and col%2==0:
                    t=Wall(row, col)
                elif row%2<>0 and col%2<>0:
                    t=Room(row, col)
                #elif row%2==0:
                #    t=random.choice([Wall(row,col),Door(row, col)])
                #elif col%2==0:
                #    t=Door(row, col)
                else:
                    t = random.choice([Joint(row,col), Door(row,col), Door(row,col), Wall(row,col)])                    
                self.add_widget(t)
                matrix_row.append(t)
            self.matrix.append(matrix_row)

        #for row in self.matrix:
        #   print [tile.row_col for tile in row]


        




class Tile(Image):
    def __init__(self, row, col, **kwargs):
        self.row_col = row, col
        
        super(Tile, self).__init__(**kwargs)
        
    def on_touch_down(self, touch):
        if self.collide_point(*touch.pos):
            print("touch {}".format(self.row_col))
            
class Room(Widget):
    def __init__(self, row, col, **kwargs):
        self.row_col = row, col
        super(Room, self).__init__(**kwargs)   


class Joint(Widget):
    def __init__(self, row, col, **kwargs):
        self.row_col = row, col
        super(Joint, self).__init__(**kwargs)  

class Wall(Tile):
    def __init__(self, row, col, **kwargs):
        super(Wall, self).__init__(row, col, **kwargs)          

class Door(Tile):
    def __init__(self, row, col, **kwargs):
        super(Door, self).__init__(row, col, **kwargs)  
        self.opacity=0.2
        self.animation = Animation()
    
    def on_touch_down(self, touch):
        if self.collide_point(*touch.pos):
            if self not in self.parent.doors:
                self.close()
                self.parent.doors.append(self)
                if len(self.parent.doors)>3:
                    self.parent.doors.pop(0).open()
        
    def open(self):
        #print 'open', self.row_col
        self.animation = Animation(opacity=0.2, duration=0.5)
        self.animation.cancel_all(self)
        self.animation.start(self)
        
    def close(self):
        #print 'close', self.row_col
        self.animation = Animation(opacity=1, duration=0.5)
        self.animation.cancel_all(self)
        self.animation.start(self)     
        

class Character(Image):
    def __init__(self, board_pos, room_size, joint_size, x, y, **kwargs):
        #print 'new charater', x, y
        self.row = BOARD_ROWS - (y*2)
        self.col = x*2 - 1
        self.size_hint=None, None
        self.room_size=room_size
        self.joint_size=joint_size
        self.size=room_size, room_size
        self.board_pos=board_pos
        self.pos = (self.board_pos[0]+(self.col/2+1)*self.joint_size+(self.col/2)*self.room_size),(self.board_pos[1]+((BOARD_ROWS-self.row)/2)*self.joint_size+((BOARD_ROWS-self.row)/2-1)*self.room_size)
        self.target_y=None
        self.animation = Animation()
        self.animation.bind(on_complete=self.go_next)
        self.animation.start(self)
        super(Character, self).__init__(**kwargs)
        
                

    def go_next(self, instance, value):
        next_pos = [[-1,0],[1,0],[0,-1],[0,1]]
        if self.col > self.target_y:
            next_pos+=[[0,-1],[0,-1]]
        elif self.col < self.target_y:
            next_pos+=[[0,+1],[0,+1]]
        
        next_row, next_col=random.choice(next_pos)
        
        check_next=self.parent.board.matrix[self.row+next_row][self.col+next_col]

        if check_next.__class__.__name__=='Wall':
            next_row = next_col = 0
        elif check_next.__class__.__name__=='Door' and check_next.opacity==1:
            next_row = next_col = 0
        self.row+=2*next_row
        self.col+=2*next_col

        self.animation.cancel_all(self, 'pos')
        self.animation = Animation(pos=((self.board_pos[0]+(self.col/2+1)*self.joint_size+(self.col/2)*self.room_size),(self.board_pos[1]+((BOARD_ROWS-self.row)/2)*self.joint_size+((BOARD_ROWS-self.row)/2-1)*self.room_size)),duration=0.5)
        self.animation.bind(on_complete=self.go_next)
        self.animation.start(self)
        
class Hero(Character):
    def __init__(self, board_pos, room_size, joint_size, x, y, **kwargs):
        super(Hero, self).__init__(board_pos, room_size, joint_size, x, y, **kwargs)
        self.target_y=ROOMS_IN_A_ROW*2
        
class Monster(Character):
    def __init__(self, board_pos, room_size, joint_size, x, y, **kwargs):
        super(Monster, self).__init__(board_pos, room_size, joint_size, x, y, **kwargs)     
        self.target_y=ROOMS_IN_A_ROW
        self.can_move=True
    '''
    def go_next(self, instance, value):        
        self.can_move=True
    
    def move(self, direction):
        if self.can_move:
            self.can_move=False
            next_row, next_col=direction
            
            check_next=self.parent.board.matrix[self.row+next_row][self.col+next_col]
            if check_next.__class__.__name__=='Wall':
                next_row = next_col = 0
            elif check_next.__class__.__name__=='Door' and check_next.opacity==1:
                next_row = next_col = 0
        
            self.row+=2*next_row
            self.col+=2*next_col
            self.animation.cancel_all(self, 'pos')
            self.animation = Animation(pos=((self.board_pos[0]+(self.col/2+1)*self.joint_size+(self.col/2)*self.room_size),(self.board_pos[1]+((BOARD_ROWS-self.row)/2)*self.joint_size+((BOARD_ROWS-self.row)/2-1)*self.room_size)),duration=0.5)
            self.animation.bind(on_complete=self.go_next)
            self.animation.start(self)
    '''
    def move(self, direction):
        self.target_y = direction
        
class Princess(Character):
    def __init__(self, board_pos, room_size, joint_size, x, y, **kwargs):
        super(Princess, self).__init__(board_pos, room_size, joint_size, x, y, **kwargs)             
        self.target_y=ROOMS_IN_A_ROW*3/2
            
class Restart(Button):
    def __init__(self, **kwargs):
        super(Restart, self).__init__(**kwargs) 

    
    
    
class GameLayout(FloatLayout):
    def __init__(self, **kwargs):
        super(GameLayout, self).__init__(**kwargs)   
        self.board=None
        self.princess=None
        self.monster=None
        self.hero=[]
        self.status='logo' # 'intro', 'starting', 'play', 'end'
        self.intro_counter=0
        self.add_widget(Image(source='screen_forkedtaillogo.png'))



   
    def update(self, dt):
        if self.status=='intro':
            if self.intro_counter>0 and self.intro_counter<60*6:
                self.intro_counter+=1
                try:
                    self.children[self.intro_counter/60].opacity=1
                except IndexError:
                    pass
                
        elif self.status=='play':
            if self.princess==None:
                p = Princess([int(p) for p in self.board.pos], self.board.room_size, self.board.joint_size, ROOMS_IN_A_ROW,random.randrange(1,ROOMS_IN_A_COL))
                self.princess=p
                self.add_widget(p)  
            if self.monster==None:
                p = Monster([int(p) for p in self.board.pos], self.board.room_size, self.board.joint_size, ROOMS_IN_A_ROW/2,random.randrange(1,ROOMS_IN_A_COL))
                self.monster=p
                self.add_widget(p) 
            if len(self.hero)<1:
                p=Hero([int(p) for p in self.board.pos], self.board.room_size, self.board.joint_size, 1,random.randrange(1,ROOMS_IN_A_COL))
                self.hero.append(p)
                self.add_widget(p)

        
            if self.princess.collide_widget(self.monster):
                self.princess.animation.cancel_all(self.princess)
                self.princess.color=[1,0,0,1]
                print 'princess killed'
                self.lose(killed=True)
            else:
                for h in self.hero:
                    if h.collide_widget(self.monster):#(self.monster.row, self.monster.col) == (h.row, h.col):
                        h.animation.cancel_all(h)
                        h.color=[1,0,0,1]
                        print 'hero killed'
                        self.hero.remove(h)
                    elif h.collide_widget(self.princess):#(self.princess.row, self.princess.col) == (h.row, h.col):
                        self.princess.animation.cancel_all(self.princess)
                        h.color=[0,1,0,1]
                        self.princess.color=[0,1,0,1]
                        print 'princess free'
                        self.lose(killed=False)
                        break
        
        
    def start(self, restart=False):
        if restart:
            self.clear_widgets()
        self.board=Board(self.size)
        self.add_widget(self.board)
        #restart_btn = Button(text='RESTART', size_hint=(None,None))
        #restart_btn.bind(on_press=self.start)
        #self.add_widget(restart_btn) 
        
    def spawn_hero(self, dt):
        if self.status=='play':#self.board<>None and self.in_game:
            p=Hero([int(p) for p in self.board.pos], self.board.room_size, self.board.joint_size, 1,random.randrange(1,ROOMS_IN_A_COL))
            self.hero.append(p)
            self.add_widget(p)    

    def on_touch_down(self, touch):
        if self.status=='logo':
            self.clear_widgets()
            self.status='intro'
            self.add_widget(Label(text='She is\nthe princess', font_size='40sp', pos_hint={'center_x': 0.8, 'center_y': 0.7}, opacity=1))
            self.add_widget(Label(text='You are the ugly monster. You can\'t let him in here\nClick on doors to block hero (max 3 doors)\nClick on left/right side of the screen to\nincrease chance to move monster in that direction\nSurvive for 160 seconds', font_size='20sp', pos_hint={'center_x': 0.5, 'center_y': 0.3}, opacity=0))
            self.add_widget(Image(source='monster_left.png', pos_hint={'center_x': 0.5, 'center_y': 0.5}, opacity=0))  
            self.add_widget(Label(text='He is\nthe hero', font_size='40sp', pos_hint={'center_x': 0.2, 'center_y': 0.7}, opacity=0))
            self.add_widget(Image(source='hero.png', pos_hint={'center_x': 0.2, 'center_y': 0.5}, opacity=0))
            self.add_widget(Image(source='princess.png', pos_hint={'center_x': 0.8, 'center_y': 0.5}, opacity=0))            
            self.intro_counter=1   
            return
        elif self.status=='intro':
            if self.intro_counter>=60*6:
                self.clear_widgets()
                self.board=Board(self.size)
                self.add_widget(self.board)
                #self.add_widget(TimerLabel(text='time left: '+str(self.spawn_seconds*20), font_size='40sp', pos_hint={'center_x': 0.8, 'center_y': 0.9}))                
                self.status='play'
                return        
        elif self.status=='play':
            if touch.pos[0]<self.board.pos[0]:
                print("going left")
                self.monster.move(1)
                self.monster.source='monster_left.png'
            elif touch.pos[0]>self.board.pos[0]+self.board.board_size[0]:
                print("going right")     
                self.monster.move(ROOMS_IN_A_ROW*2)
                self.monster.source='monster_right.png'
            '''
            elif touch.pos[1]<self.board.pos[1]:
                print("down")        
                self.monster.move([1,0])                
            elif touch.pos[1]>self.board.pos[1]+self.board.board_size[1]:
                print("up")           
                self.monster.move([-1,0])                
            '''
            return super(GameLayout, self).on_touch_down(touch)
        elif self.status=='end':
            self.clear_widgets()
            self.board=self.princess=self.monster=None
            self.hero=[]
            self.board=Board(self.size)
            self.add_widget(self.board)
            
            self.status='play'
        else:
            print self.status

    def win(self, dt):
        for c in self.children:
            try:
                c.animation.cancel_all(c)
            except AttributeError:
                pass
        self.remove_widget(self.board)
        self.add_widget(Label(text='You Win', font_size='40sp', pos_hint={'center_x': 0.5, 'center_y': 0.8}))
        self.status='end'#in_game=False
        

    def lose(self, killed):
        for c in self.children:
            try:
                c.animation.cancel_all(c)
            except AttributeError:
                pass
        self.remove_widget(self.board)        
        if killed:
            text='You Lose, you killed Her' 
        else:
            text='You Lose: you let Him in here' 
        self.add_widget(Label(text=text, font_size='40sp', pos_hint={'center_x': 0.5, 'center_y': 0.8}))
        self.status='end'#in_game=False

        
class TimerLabel(Label):
    def __init__(self, **kwargs):
        super(TimerLabel, self).__init__(**kwargs)
    
      
class Doors(App):
    def build(self):
        self.icon = ''
        self.title = 'Doors'    
        self.game = GameLayout()
        self.game.spawn_seconds=8
        
        Clock.schedule_interval(self.game.update, 1.0 / 60.0)
        Clock.schedule_interval(self.game.spawn_hero, self.game.spawn_seconds / 1.0)
        Clock.schedule_interval(self.reduce_spawn, 20 / 1.0)
        return self.game
    
    def reduce_spawn(self, dt):
        if self.game.spawn_seconds>1:
            self.game.spawn_seconds-=1
        elif self.game.status=='play':
            self.game.win(0)
        
        
if __name__ in ('__main__', '__android__'):
    Doors().run() 
    
    
    
